@@ -20,6 +20,7 @@ from zaqar.common import decorators
 from zaqar.common.transport.wsgi import helpers as wsgi_helpers
 from zaqar.i18n import _
 from zaqar.storage import errors as storage_errors
+from zaqar.transport import acl
 from zaqar.transport import utils
 from zaqar.transport import validation
 from zaqar.transport.wsgi import errors as wsgi_errors
@@ -27,17 +28,28 @@ from zaqar.transport.wsgi import utils as wsgi_utils
 
 LOG = logging.getLogger(__name__)
 
-MESSAGE_POST_SPEC = (('ttl', int, None), ('body', '*', None))
-
 
 class CollectionResource(object):
 
-    __slots__ = ('_message_controller', '_wsgi_conf', '_validate')
+    __slots__ = (
+        '_message_controller',
+        '_queue_controller',
+        '_wsgi_conf',
+        '_validate',
+        '_default_message_ttl',
+        '_encryptor'
+    )
+
+    def __init__(self, wsgi_conf, validate,
+                 message_controller, queue_controller,
+                 default_message_ttl, encryptor_factory):
 
-    def __init__(self, wsgi_conf, validate, message_controller):
         self._wsgi_conf = wsgi_conf
         self._validate = validate
         self._message_controller = message_controller
+        self._queue_controller = queue_controller
+        self._default_message_ttl = default_message_ttl
+        self._encryptor = encryptor_factory.getEncryptor()
 
     # ----------------------------------------------------------------------
     # Helpers
@@ -52,10 +64,14 @@ class CollectionResource(object):
                 message_ids=ids,
                 project=project_id)
 
+            queue_meta = self._queue_controller.get_metadata(queue_name,
+                                                             project_id)
         except validation.ValidationFailed as ex:
             LOG.debug(ex)
             raise wsgi_errors.HTTPBadRequestAPI(str(ex))
-
+        except storage_errors.QueueDoesNotExist:
+            LOG.exception('Queue name "%s" does not exist', queue_name)
+            queue_meta = None
         except Exception:
             description = _(u'Message could not be retrieved.')
             LOG.exception(description)
@@ -66,7 +82,14 @@ class CollectionResource(object):
         if not messages:
             return None
 
-        return [wsgi_utils.format_message_v1(m, base_path) for m in messages]
+        # Decrypt messages
+        if queue_meta and queue_meta.get('_enable_encrypt_messages', False):
+            self._encryptor.message_decrypted(messages)
+
+        messages = [wsgi_utils.format_message_v1_1(m, base_path, m['claim_id'])
+                    for m in messages]
+
+        return {'messages': messages}
 
     def _get(self, req, project_id, queue_name):
         client_uuid = wsgi_helpers.get_client_uuid(req)
@@ -78,8 +101,26 @@ class CollectionResource(object):
         req.get_param_as_int('limit', store=kwargs)
         req.get_param_as_bool('echo', store=kwargs)
         req.get_param_as_bool('include_claimed', store=kwargs)
+        req.get_param_as_bool('include_delayed', store=kwargs)
 
         try:
+            queue_meta = {}
+            try:
+                # NOTE(cdyangzhenyu): In order to determine whether the
+                # queue has a delay attribute, the metadata of the queue
+                # is obtained here. This may have a little performance impact.
+                # So maybe a refactor is needed in the future.
+                queue_meta = self._queue_controller.get_metadata(queue_name,
+                                                                 project_id)
+            except storage_errors.DoesNotExist:
+                LOG.exception('Queue name "%s" does not exist', queue_name)
+            queue_delay = queue_meta.get('_default_message_delay')
+            if not queue_delay:
+                # NOTE(cdyangzhenyu): If the queue without the metadata
+                # attribute _default_message_delay, we don't filter
+                # for delay messages.
+                kwargs['include_delayed'] = True
+
             self._validate.message_listing(**kwargs)
             results = self._message_controller.list(
                 queue_name,
@@ -91,13 +132,17 @@ class CollectionResource(object):
             cursor = next(results)
             messages = list(cursor)
 
+            # Decrypt messages
+            if queue_meta.get('_enable_encrypt_messages', False):
+                self._encryptor.message_decrypted(messages)
+
         except validation.ValidationFailed as ex:
             LOG.debug(ex)
             raise wsgi_errors.HTTPBadRequestAPI(str(ex))
 
-        except storage_errors.DoesNotExist as ex:
+        except storage_errors.QueueDoesNotExist as ex:
             LOG.debug(ex)
-            raise wsgi_errors.HTTPNotFound(str(ex))
+            messages = None
 
         except Exception:
             description = _(u'Messages could not be listed.')
@@ -105,22 +150,28 @@ class CollectionResource(object):
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
         if not messages:
-            return None
+            messages = []
 
-        # Found some messages, so prepare the response
-        kwargs['marker'] = next(results)
-        base_path = req.path.rsplit('/', 1)[0]
-        messages = [wsgi_utils.format_message_v1(
-            m, base_path) for m in messages]
+        else:
+            # Found some messages, so prepare the response
+            kwargs['marker'] = next(results)
+            base_path = req.path.rsplit('/', 1)[0]
+            messages = [wsgi_utils.format_message_v1_1(m, base_path,
+                                                       m['claim_id'])
+                        for m in messages]
 
-        return {
-            'messages': messages,
-            'links': [
+        links = []
+        if messages:
+            links = [
                 {
                     'rel': 'next',
                     'href': req.path + falcon.to_query_str(kwargs)
                 }
             ]
+
+        return {
+            'messages': messages,
+            'links': links
         }
 
     # ----------------------------------------------------------------------
@@ -128,24 +179,61 @@ class CollectionResource(object):
     # ----------------------------------------------------------------------
 
     @decorators.TransportLog("Messages collection")
+    @acl.enforce("messages:create")
     def on_post(self, req, resp, project_id, queue_name):
         client_uuid = wsgi_helpers.get_client_uuid(req)
-
         try:
+            # NOTE(flwang): Replace 'exists' with 'get_metadata' won't impact
+            # the performance since both of them will call
+            # collection.find_one()
+            queue_meta = None
+            try:
+                queue_meta = self._queue_controller.get_metadata(queue_name,
+                                                                 project_id)
+            except storage_errors.DoesNotExist:
+                self._validate.queue_identification(queue_name, project_id)
+                self._queue_controller.create(queue_name, project=project_id)
+                # NOTE(flwang): Queue is created in lazy mode, so no metadata
+                # set.
+                queue_meta = {}
+
+            queue_max_msg_size = queue_meta.get('_max_messages_post_size')
+            queue_default_ttl = queue_meta.get('_default_message_ttl')
+            queue_delay = queue_meta.get('_default_message_delay')
+            queue_encrypted = queue_meta.get('_enable_encrypt_messages', False)
+
+            if queue_default_ttl:
+                message_post_spec = (('ttl', int, queue_default_ttl),
+                                     ('body', '*', None),)
+            else:
+                message_post_spec = (('ttl', int, self._default_message_ttl),
+                                     ('body', '*', None),)
+            if queue_delay:
+                message_post_spec += (('delay', int, queue_delay),)
             # Place JSON size restriction before parsing
-            self._validate.message_length(req.content_length)
+            self._validate.message_length(req.content_length,
+                                          max_msg_post_size=queue_max_msg_size)
         except validation.ValidationFailed as ex:
             LOG.debug(ex)
             raise wsgi_errors.HTTPBadRequestAPI(str(ex))
 
-        # Deserialize and validate the request body
+        # Deserialize and validate the incoming messages
         document = wsgi_utils.deserialize(req.stream, req.content_length)
-        messages = wsgi_utils.sanitize(document, MESSAGE_POST_SPEC,
+
+        if 'messages' not in document:
+            description = _(u'No messages were found in the request body.')
+            raise wsgi_errors.HTTPBadRequestAPI(description)
+
+        messages = wsgi_utils.sanitize(document['messages'],
+                                       message_post_spec,
                                        doctype=wsgi_utils.JSONArray)
 
         try:
             self._validate.message_posting(messages)
 
+            if queue_encrypted:
+                self._encryptor.message_encrypted(messages)
+
             message_ids = self._message_controller.post(
                 queue_name,
                 messages=messages,
@@ -175,68 +263,117 @@ class CollectionResource(object):
         resp.location = req.path + '?ids=' + ids_value
 
         hrefs = [req.path + '/' + id for id in message_ids]
-
-        # NOTE(kgriffs): As of the Icehouse release, drivers are
-        # no longer allowed to enqueue a subset of the messages
-        # submitted by the client; it's all or nothing. Therefore,
-        # 'partial' is now always False in the v1.0 API, and the
-        # field has been removed in v1.1.
-        body = {'resources': hrefs, 'partial': False}
-
+        body = {'resources': hrefs}
         resp.body = utils.to_json(body)
         resp.status = falcon.HTTP_201
 
     @decorators.TransportLog("Messages collection")
+    @acl.enforce("messages:get_all")
     def on_get(self, req, resp, project_id, queue_name):
-        resp.content_location = req.relative_uri
-
         ids = req.get_param_as_list('ids')
+
         if ids is None:
             response = self._get(req, project_id, queue_name)
+
         else:
             response = self._get_by_id(req.path.rsplit('/', 1)[0], project_id,
                                        queue_name, ids)
 
         if response is None:
-            resp.status = falcon.HTTP_204
-            return
+            # NOTE(TheSriram): Trying to get a message by id, should
+            # return the message if its present, otherwise a 404 since
+            # the message might have been deleted.
+            msg = _(u'No messages with IDs: {ids} found in the queue {queue} '
+                    u'for project {project}.')
+            description = msg.format(queue=queue_name, project=project_id,
+                                     ids=ids)
+            raise wsgi_errors.HTTPNotFound(description)
 
-        resp.body = utils.to_json(response)
+        else:
+            resp.body = utils.to_json(response)
         # status defaults to 200
 
     @decorators.TransportLog("Messages collection")
+    @acl.enforce("messages:delete_all")
     def on_delete(self, req, resp, project_id, queue_name):
-        # NOTE(zyuan): Attempt to delete the whole message collection
-        # (without an "ids" parameter) is not allowed
-        ids = req.get_param_as_list('ids', required=True)
-
+        ids = req.get_param_as_list('ids')
+        claim_ids = None
+        if self._validate.get_limit_conf_value('message_delete_with_claim_id'):
+            claim_ids = req.get_param_as_list('claim_ids')
+        pop_limit = req.get_param_as_int('pop')
         try:
-            self._validate.message_listing(limit=len(ids))
-            self._message_controller.bulk_delete(
-                queue_name,
-                message_ids=ids,
-                project=project_id)
+            self._validate.message_deletion(ids, pop_limit, claim_ids)
 
         except validation.ValidationFailed as ex:
             LOG.debug(ex)
             raise wsgi_errors.HTTPBadRequestAPI(str(ex))
 
+        if ids:
+            resp.status = self._delete_messages_by_id(queue_name, ids,
+                                                      project_id, claim_ids)
+
+        elif pop_limit:
+            resp.status, resp.body = self._pop_messages(queue_name,
+                                                        project_id,
+                                                        pop_limit)
+
+    def _delete_messages_by_id(self, queue_name, ids, project_id,
+                               claim_ids=None):
+        try:
+            self._message_controller.bulk_delete(
+                queue_name,
+                message_ids=ids,
+                project=project_id,
+                claim_ids=claim_ids)
+
         except Exception:
             description = _(u'Messages could not be deleted.')
             LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
-        resp.status = falcon.HTTP_204
+        return falcon.HTTP_204
+
+    def _pop_messages(self, queue_name, project_id, pop_limit):
+        try:
+            LOG.debug(u'POP messages - queue: %(queue)s, '
+                      u'project: %(project)s',
+                      {'queue': queue_name, 'project': project_id})
+
+            messages = self._message_controller.pop(
+                queue_name,
+                project=project_id,
+                limit=pop_limit)
+
+        except Exception:
+            description = _(u'Messages could not be popped.')
+            LOG.exception(description)
+            raise wsgi_errors.HTTPServiceUnavailable(description)
+
+        # Prepare response
+        if not messages:
+            messages = []
+        body = {'messages': messages}
+        body = utils.to_json(body)
+
+        return falcon.HTTP_200, body
 
 
 class ItemResource(object):
 
-    __slots__ = '_message_controller'
+    __slots__ = (
+        '_message_controller',
+        '_queue_controller',
+        '_encryptor'
+    )
 
-    def __init__(self, message_controller):
+    def __init__(self, message_controller, queue_controller,
+                 encryptor_factory):
         self._message_controller = message_controller
+        self._queue_controller = queue_controller
+        self._encryptor = encryptor_factory.getEncryptor()
 
     @decorators.TransportLog("Messages item")
+    @acl.enforce("messages:get")
     def on_get(self, req, resp, project_id, queue_name, message_id):
         try:
             message = self._message_controller.get(
@@ -244,6 +381,12 @@ class ItemResource(object):
                 message_id,
                 project=project_id)
 
+            queue_meta = self._queue_controller.get_metadata(queue_name,
+                                                             project_id)
+            # Decrypt messages
+            if queue_meta.get('_enable_encrypt_messages', False):
+                self._encryptor.message_decrypted([message])
+
         except storage_errors.DoesNotExist as ex:
             LOG.debug(ex)
             raise wsgi_errors.HTTPNotFound(str(ex))
@@ -253,13 +396,17 @@ class ItemResource(object):
             LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
-        resp.content_location = req.relative_uri
-        message = wsgi_utils.format_message_v1(
-            message, req.path.rsplit('/', 2)[0])
+        # Prepare response
+        message['href'] = req.path
+        message = wsgi_utils.format_message_v1_1(message,
+                                                 req.path.rsplit('/', 2)[0],
+                                                 message['claim_id'])
+
         resp.body = utils.to_json(message)
         # status defaults to 200
 
     @decorators.TransportLog("Messages item")
+    @acl.enforce("messages:delete")
     def on_delete(self, req, resp, project_id, queue_name, message_id):
         error_title = _(u'Unable to delete')
 
